use std::cmp::Ordering;

use binrw::{BinRead, BinResult};
use bitreader::{BitReader, BitReaderError};

fn parse_memory_zone_size_bitreader(bytes: &[u8; 5]) -> Result<u32, BitReaderError> {
    let mut reader = BitReader::new(bytes);

    let zone_size_bit_count = match reader.read_bool()? {
        true => 7,
        false => reader.read_u8(7)? * 8,
    };
    Ok(reader.read_u32(zone_size_bit_count)?)
}

#[binrw::parser(reader)]
fn parse_memory_zone_size() -> BinResult<u32> {
    let mut value: u8 = 0;
     

    reader.read_exact(std::slice::from_mut(&mut value));

    // first bit s 0 <=> value < 128
    match value.cmp(&128) {
        Ordering::Less => {
            Ok(value.into())
        }
        Ordering::Equal | Ordering::Greater => {
            let zone_size_encoding_byte_count = value - 128;
            let mut buf: Vec<u8> = vec![0; zone_size_encoding_byte_count.into()];
            reader.read_exact(&mut buf);

            let mut result = 0u32;

            for (i, &byte) in buf.iter().enumerate() {
                result += (byte as u32) << (8 * i);
            }

            Ok(result)
        }
    }
}

#[derive(BinRead)]
struct MemoryZone {
    #[br(parse_with = parse_memory_zone_size)]
    memory_size: u32,
}

enum TypeIdentificationStructure {
    NumeroAdeliCabinet,
    NumeroFINESS,
    NumeroSIREN,
    NumeroSIRET,
    NumeroRPPSCabinet,
}

pub enum TypeDIdentificationNationale {
    NumeroAdeli,
    NumeroAdeliCabinetNumeroEmploye,
    NumeroDRASS,
    NumeroFINESSNumeroEmploye,
    NumeroSIRENNumeroEmploye,
    NumeroSIRETNumeroEmploye,
    NumeroRPPSCabinetNumeroEmploye,
    NumeroRPPS,
    /// N° Etudiant Médecin type ADELI sur 9 caractères (information transmise par l’ANS)
    NumeroEtudiantMedecin,
}
pub(crate) enum TypeCartePS {
    /// Carte de Professionnel de Santé (CPS)
    CarteDeProfessionnelSante,
    /// Carte de Professionnel de Santé en Formation (CPF)
    CarteDeProfessionnelSanteEnFormation,
    /// Carte de Personnel d'Établissement de Santé (CDE/CPE)
    CarteDePersonnelEtablissementSante,
    /// Carte de Personnel Autorisé (CDA/CPA)
    CarteDePersonnelAutorise,
    /// Carte de Personne Morale
    CarteDePersonneMorale,
}
